#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/mman.h>
#include <errno.h>
#include <signal.h>
#include <ctype.h>
#include <time.h>
#include "wusbxx_dll.h" /* the header of the shared library */
#include "daq.h"

//#define DEBUG /* vkljuci dodatni izpis */
#ifdef DEBUG
#  define DBG(X) X
#  define DBGFUNI(X) printf(">>> %s -> %d\n",#X,(X))
# else
#  define DBG(X)
#  define DBGFUNI(X) X
#endif

/* definiram lokacije enot*/
#define NADC 15         /* ADC */
#define NADCCH 10
int ctrlc=0;
char *ccserial="CC0126";

int daq::connect(){
// odpri daq
    WUSBXX_load(NULL);
    WUSBXX_open(ccserial);
    printf("daq::connect()\n");
    return 0;
}

int daq::init(){

    int i;  
    long k;

    DBGFUNI(xxusb_register_write(udev,1,0x0)); // Stop DAQ mode
    while (xxusb_usbfifo_read(udev, (int*) stackdump,BUFF_L,100)>0); 
  
    CCCZ;
    CCCC;
    CREM_I;

 
// create command stack
    k=1;
    for(i=0;i<NADCCH;i++) stackwrite[k++]=NAF(NADC,i,0);
    stackwrite[k++]=NAF(NADC,0,9);
    stackwrite[k++]=NAFS(0,0,16); /* insert next word to data */
    stackwrite[k++]=0xfafb; /* event termination word */
    stackwrite[0]=k-1;
// upload stack #2
    xxusb_stack_write(udev,0x2,(long int *)stackwrite);
    xxusb_stack_read(udev,0x2,(long int *) stackdata);
    DBG(for(i=0;i<k;i++) printf("0x%04x\n",stackdata[i]);)
    
    CAMAC_LED_settings(udev, 1,1,0,0);
    CAMAC_register_read(udev,0,&k);
    printf("Firmware ID -> 0x%08X\n",k);
    CAMAC_register_read(udev,1,&k);
    printf("Global Mode -> 0x%08X\n",k);
    k=(k&0xF000)|0x0005; /* set buffer length: n=0..6 -> 0x10000 >> n, n=7 -> single event */
    CAMAC_register_write(udev,1,k);
    CAMAC_register_write(udev,2,0x80); // wait 0x80 us after trigger
    CAMAC_register_write(udev,3,0x0); // Scaler Readout Control Register
    CAMAC_register_write(udev,9,0x0); // Lam Mask Register
    CAMAC_register_write(udev,14,0x0); // USB Bulk Transfer Setup Register

    //    CAMAC_DGG(udev,1,2,3,0,200,0,0);
    //    CAMAC_DGG(udev,0,0,0,0,100,0,0);
    CAMAC_register_write(udev,5,(0x06<<16)+(0x04<<8)+0x00); // output
    CAMAC_register_write(udev,6,(0x01<<24)+(0x01<<16)+(0x0<<8)+0x0); // SCLR & DGG
    CAMAC_register_write(udev,7,(150<<16)+0); // output 100=1us
    CAMAC_register_write(udev,8,(14000<<16)+0); // output
    CAMAC_register_write(udev,13,0); // output
    
    return 0;
}

int daq::start(){
    xxusb_register_write(udev,1,0x1); // Start DAQ mode
    return 0;
}

int daq::stop(){
    xxusb_register_write(udev,1,0x0); // Stop DAQ mode
    while (xxusb_usbfifo_read(udev,(int *)stackdump,BUFF_L,30)>0);
    return 0;
}

int daq::event(unsigned int *data, int maxn){
    int i,ib,count;
    int events,evsize;
    short ret;

    ib=0;
    ret=xxusb_usbfifo_read(udev,(int *) stackdata,BUFF_L,500);
    events=stackdata[ib++];
    DBG(printf("ret=%d,events=0x%08x\n",ret,events);)
      if ((ret<0)||(ret!=((NADCCH*4+4)*events+4))) return 0;

    count=0;
    while (ib<(ret/2-1)){
      evsize = stackdata[ib++]&0xffff;
      DBG(printf("Event:%d EvSize:%d\n", events, evsize);)
      for (int i=0;i<NADCCH;i++,ib++)  data[count++] =stackdata[ib++]&0xffff;
      if (stackdata[ib++]!=0xfafb){
        printf("Error!\n");
        return 0;
      }
      events--;
      if (fStop) return 0;
    } 
    if (stackdata[ib++]!=0xffff){
      printf("Error!\n");
      return 0;
    }
    return count;
}
  
int daq::disconnect(){ 
// zapri daq
    WUSBXX_close();
    printf("daq::disconnect()\n");
    return 0;
}

daq::daq(){
fStop=0;
connect();
init();
}

daq::~daq(){
disconnect();
}

#ifdef MAIN
void CatchSig (int signumber)
{
   ctrlc = 1;
}


int main (int argc, char **argv){
  
  int neve=1000000;
  char *fname="test.dat";
  if (argc==1) {
    printf("Uporaba: %s stevilo_dogodkov ime_datoteke\n",argv[0]);
    printf("Meritev prekini s Ctrl-C, ce je nabranih dogodkov ze dovolj\n");
    exit(0);
  } 
  if (argc>1) neve  = atoi(argv[1]);
  if (argc>2) fname = argv[2];
  
  
// intercept routine
  if (signal (SIGINT, CatchSig) == SIG_ERR) perror ("sigignore");
   
#define BSIZE 10000
  int i,ieve,nc,nb;
  int hdr[4]={1,(NADCCH+4)*sizeof(int)};
  unsigned short adc;
  unsigned int data[BSIZE];
  daq *d= new daq();

// odpremo datoteko za pisanje
  FILE *fp=fopen(fname,"w");


//  d->init();
  d->start();
  ieve=0;
  while((ieve<neve)&&(!ctrlc)){
    nc=d->event(data,BSIZE);
    nb=0;
    while((nc>0)&&(ieve++<neve)&&(!ctrlc)){
      // zapis v datoteko   
      hdr[2]=time(NULL);
      hdr[3]=ieve;
      fwrite(hdr,sizeof(int),4 ,fp); 
      fwrite(&data[nb],sizeof(int),NADCCH,fp);
      DBG(
        for(i=0;i<NADCCH;i++){
          adc=data[nb+i]&0xFFFF;	
          printf("nev=%4d %d. ADC data=%d\n",ieve,i,adc);
        }
      )
      nb+=NADCCH;
      nc-=NADCCH;
      if (!(ieve%1000)) printf("event no. -> %d\n",ieve);
    };
  };
   
  d->stop();
  fclose(fp);
  printf("Podatki so v datoteki %s\n", fname);
  delete d;

  return 0;
}
#endif
