#include <stdlib.h>
#include <stdio.h>
#include "TH1F.h"
#include "TCanvas.h"
#include "TStyle.h"

#define MAXCH 32
TH1F* gHisto[MAXCH];
TH1F* gHistoSum[2];
TH1F* gHistoXavg[2];
TH1F* gHistoYavg[2];

int xcoordinates[4] = {-1, 1, -1, 1};
int ycoordinates[4] = {-1, -1, 1, 1};

int init(){
  gStyle->SetOptFit(1011);

  for (int i=0;i<MAXCH;i++){
    char hname[50];
    char hn[50];
    if (i<16){
      sprintf(hname,"ADC Ch. %d;ADC;N",i);
      sprintf(hn,"ach%d",i);
    } else {
      sprintf(hname,"TDC Ch. %d;TDC;N",i-16);
      sprintf(hn,"tch%d",i-16);
    }

    gHisto[i]   = new TH1F(hn,hname,1200,-0.5,1199.5);
  }  
 gHistoSum[0]   = new TH1F("hsum0","hsum0",4200,-0.5,4199.5);
 gHistoSum[1]   = new TH1F("hsum1","hsum1",4200,-0.5,4199.5);
 return 0;
}

int readtdcfile(char *fname)
{
  
  init();
  
  FILE *fp=fopen(fname,"r");

  const int bsize = 10000;
  unsigned int buf[10000];
  int hdr[4];

  if (fp){

    while (!feof(fp) ){

      int nitems = fread(hdr, sizeof(int), 4, fp);
 
      if (nitems == 4) {
	// napolni histograme
	int nb = hdr[1] - 4*sizeof(int);

	if ( nb > bsize) {
	  nb = bsize;
	  printf ("Error bsize %d nb%d\n",bsize,nb);
	}

	int nint = nb / sizeof(int);
	int n=hdr[3];

	nitems = fread(buf, sizeof(int), nb, fp);
	int sum[2] = {0,0};

	if (nitems == nb) {
	  
	  for (int i=0; i<8 ; i++) {
	    unsigned short adc = buf[i]&0xFFFF;
	    if (i<4) sum[0]+=adc; else sum[1]+=adc;
	    gHisto[i]->Fill(adc);
	  }

	  gHistoXavg[0]->Fill(buf[0]*xcoordinates[0]+
			      buf[1]*xcoordinates[1]+
			      buf[2]*xcoordinates[2]+
			      buf[3]*xcoordinates[3]);

	  gHistoYavg[0]->Fill(buf[0]*ycoordinates[0]+
			      buf[1]*ycoordinates[1]+
			      buf[2]*ycoordinates[2]+
			      buf[3]*ycoordinates[3]);

	  gHistoXavg[1]->Fill(buf[4]*xcoordinates[0]+
			      buf[5]*xcoordinates[1]+
			      buf[6]*xcoordinates[2]+
			      buf[7]*xcoordinates[3]);

	  gHistoYavg[1]->Fill(buf[4]*ycoordinates[0]+
			      buf[5]*ycoordinates[1]+
			      buf[6]*ycoordinates[2]+
			      buf[7]*ycoordinates[3]);


	  gHistoSum[0]->Fill(sum[0]);
	  gHistoSum[1]->Fill(sum[1]);

	  for (int i=8;i<nint;i++){
	    
	    unsigned short tdc =   buf[i]&0xFFFF;                   
	    unsigned short ch  =  (buf[i] >> 17 ) &0x1F;                             
	    unsigned short edge = (buf[i] >> 16 ) &0x1;
	    if (edge && ch < MAXCH) gHisto[ch+16]->Fill(tdc);
	    printf("nev=%4d %d. [ch=%2d] egde=%d data=%d\n",n,i,ch,edge,tdc);
	  }
	}
      }
    }
    fclose(fp);
    
  }
  printf("Konec\n");
  //if (buf)  delete buf;
  TCanvas *c1=new TCanvas("c1","c1");
  gHisto[1]->Draw();
  TCanvas *c2=new TCanvas("c2","c2");
  gHisto[2]->Draw();
  c1->Modified();
  c1->Update();
  c2->Modified();
  c2->Update();
  return 0;
}
